package hu.bme.mit.socialnetwork.graphiti;

import hu.bme.mit.socialnetwork.graphiti.features.PersonCopyFeature;
import hu.bme.mit.socialnetwork.graphiti.features.PersonDirectEditFeature;
import hu.bme.mit.socialnetwork.graphiti.features.PersonLayoutFeature;
import hu.bme.mit.socialnetwork.graphiti.features.AddAcquaintanceFeature;
import hu.bme.mit.socialnetwork.graphiti.features.AddPersonFeature;
import hu.bme.mit.socialnetwork.graphiti.features.AquaintanceCreateFeature;
import hu.bme.mit.socialnetwork.graphiti.features.PersonCreateFeature;
import hu.bme.mit.socialnetwork.graphiti.features.PersonPasteFeature;
import hu.bme.mit.socialnetwork.graphiti.features.PersonUpdateFeature;

import org.eclipse.graphiti.dt.IDiagramTypeProvider;
import org.eclipse.graphiti.features.IAddFeature;
import org.eclipse.graphiti.features.ICopyFeature;
import org.eclipse.graphiti.features.ICreateConnectionFeature;
import org.eclipse.graphiti.features.ICreateFeature;
import org.eclipse.graphiti.features.IDirectEditingFeature;
import org.eclipse.graphiti.features.ILayoutFeature;
import org.eclipse.graphiti.features.IPasteFeature;
import org.eclipse.graphiti.features.IUpdateFeature;
import org.eclipse.graphiti.features.context.IAddContext;
import org.eclipse.graphiti.features.context.ICopyContext;
import org.eclipse.graphiti.features.context.IDirectEditingContext;
import org.eclipse.graphiti.features.context.ILayoutContext;
import org.eclipse.graphiti.features.context.IPasteContext;
import org.eclipse.graphiti.features.context.IUpdateContext;
import org.eclipse.graphiti.mm.pictograms.ContainerShape;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.ui.features.DefaultFeatureProvider;

import socialnetwork.Acquaintance;
import socialnetwork.Person;

public class SocialNetworkFeatureProvider extends DefaultFeatureProvider {

	public SocialNetworkFeatureProvider(IDiagramTypeProvider dtp) {
		super(dtp);
	}

	@Override
	public IAddFeature getAddFeature(IAddContext context) {
		if (context.getNewObject() instanceof Person) {
			return new AddPersonFeature(this);
		} else if (context.getNewObject() instanceof Acquaintance) {
			return new AddAcquaintanceFeature(this);
		}
		return super.getAddFeature(context);
	}

	@Override
	public ICreateFeature[] getCreateFeatures() {
		return new ICreateFeature[] { new PersonCreateFeature(this)};
	}

	@Override
	public ICreateConnectionFeature[] getCreateConnectionFeatures() {
		return new ICreateConnectionFeature[]{new AquaintanceCreateFeature(this)};
	}

	@Override
	public IUpdateFeature getUpdateFeature(IUpdateContext context) {
		PictogramElement pe = context.getPictogramElement();
		if (pe instanceof ContainerShape) {
			Object bo = getBusinessObjectForPictogramElement(pe);
			if (bo instanceof Person) {
				return new PersonUpdateFeature(this);
			}
		}
		return super.getUpdateFeature(context);
	}

	@Override
	public ILayoutFeature getLayoutFeature(ILayoutContext context) {
		PictogramElement pe = context.getPictogramElement();
		Object bo = getBusinessObjectForPictogramElement(pe);
		if (bo instanceof Person) {
			return new PersonLayoutFeature(this);
		}
		return super.getLayoutFeature(context);
	}

	@Override
	public IDirectEditingFeature getDirectEditingFeature(
			IDirectEditingContext context) {
		PictogramElement pe = context.getPictogramElement();
		Object bo = getBusinessObjectForPictogramElement(pe);
		if (bo instanceof Person) {
			return new PersonDirectEditFeature(this);
		}
		return super.getDirectEditingFeature(context);
	}

	@Override
	public ICopyFeature getCopyFeature(ICopyContext context) {
		return new PersonCopyFeature(this);
	}

	@Override
	public IPasteFeature getPasteFeature(IPasteContext context) {
		return new PersonPasteFeature(this);
	}

	
	
}
