package hu.bme.mit.socialnetwork.graphiti.features;

import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.IAddContext;
import org.eclipse.graphiti.features.impl.AbstractAddShapeFeature;
import org.eclipse.graphiti.mm.algorithms.Polyline;
import org.eclipse.graphiti.mm.algorithms.RoundedRectangle;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.algorithms.styles.Orientation;
import org.eclipse.graphiti.mm.pictograms.ContainerShape;
import org.eclipse.graphiti.mm.pictograms.Diagram;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.mm.pictograms.Shape;
import org.eclipse.graphiti.services.Graphiti;
import org.eclipse.graphiti.services.IGaService;
import org.eclipse.graphiti.services.IPeCreateService;
import org.eclipse.graphiti.util.ColorConstant;
import org.eclipse.graphiti.util.IColorConstant;

import socialnetwork.Person;

public class AddPersonFeature extends AbstractAddShapeFeature{

	private final static IColorConstant PERSON_TEXT_FOREGROUND = IColorConstant.BLACK;
	private final static IColorConstant PERSON_FOREGROUND = new ColorConstant(98, 131, 167);
	private final static IColorConstant PERSON_BACKGROUND = new ColorConstant(187, 218, 247);
	
	public AddPersonFeature(IFeatureProvider fp) {
		super(fp);
	}

	@Override
	public boolean canAdd(IAddContext context) {
		return context.getNewObject() instanceof Person
				&& context.getTargetContainer() instanceof Diagram;
	}

	@Override
	public PictogramElement add(IAddContext context) {
		Person person = (Person) context.getNewObject();
		Diagram targetDiagram = (Diagram) context.getTargetContainer();
		
		IPeCreateService peCreateService = Graphiti.getPeCreateService();
		IGaService gaService = Graphiti.getGaService();
		
		ContainerShape containerShape = peCreateService.createContainerShape(targetDiagram, true);
		int width = 100;
		int height = 50;
		RoundedRectangle rectangle = gaService.createRoundedRectangle(containerShape, 5, 5);
		rectangle.setForeground(manageColor(PERSON_FOREGROUND));
		rectangle.setBackground(manageColor(PERSON_BACKGROUND));
		rectangle.setLineWidth(2);
		gaService.setLocationAndSize(rectangle, context.getX(), context.getY(),
				width, height);
		
		Shape lineShape = peCreateService.createShape(containerShape, false);
		Polyline line = gaService.createPlainPolyline(lineShape, new int[] {0, 20, width, 20});
		line.setForeground(manageColor(PERSON_FOREGROUND));
		line.setWidth(2);
		
		Shape textShape = peCreateService.createShape(containerShape, false);
		Text text = gaService.createText(textShape, person.getName());
		text.setForeground(manageColor(PERSON_TEXT_FOREGROUND));
		text.setHorizontalAlignment(Orientation.ALIGNMENT_CENTER);
		text.setFont(gaService.manageDefaultFont(getDiagram(), false, true));
		gaService.setLocationAndSize(text, 0, 0, width, 20);
		
		//Adding anchors for connections
		peCreateService.createChopboxAnchor(containerShape);
		
		//Linking
		if (person.eResource() == null) {
			getDiagram().eResource().getContents().add(person);
		}
		link(containerShape, person);
		link(textShape, person);
		
		//Calling the layout feature
		layoutPictogramElement(containerShape);
		
		return containerShape;
	}

}
