package hu.bme.mit.socialnetwork.graphiti.features;

import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.ICreateConnectionContext;
import org.eclipse.graphiti.features.context.impl.AddConnectionContext;
import org.eclipse.graphiti.features.impl.AbstractCreateConnectionFeature;
import org.eclipse.graphiti.mm.pictograms.Anchor;
import org.eclipse.graphiti.mm.pictograms.Connection;

import socialnetwork.Acquaintance;
import socialnetwork.Person;
import socialnetwork.SocialnetworkFactory;

public class AquaintanceCreateFeature extends
		AbstractCreateConnectionFeature {

	public AquaintanceCreateFeature(IFeatureProvider fp) {
		super(fp, "Acquaintance", "Defines an Acquaintance");
	}

	@Override
	public boolean canStartConnection(ICreateConnectionContext context) {
		Anchor source = context.getSourceAnchor();
		return getPersonFromAnchor(source) != null;
	}
	
	@Override
	public boolean canCreate(ICreateConnectionContext context) {
		Person src = getPersonFromAnchor(context.getSourceAnchor());
		Person trg = getPersonFromAnchor(context.getTargetAnchor());
		return src != null && trg != null && src != trg;
	}

	@Override
	public Connection create(ICreateConnectionContext context) {
		Anchor sourceAnchor = context.getSourceAnchor();
		Anchor targetAnchor = context.getTargetAnchor();
		Person src = getPersonFromAnchor(sourceAnchor);
		Person trg = getPersonFromAnchor(targetAnchor);
		
		if (src != null && trg != null) {
			AddConnectionContext ctx = new AddConnectionContext(sourceAnchor, targetAnchor);
			ctx.setNewObject(createAcquaintance(src, trg));
			return (Connection) getFeatureProvider().addIfPossible(ctx);
		}
		return null;
	}

	private Person getPersonFromAnchor(Anchor anchor) {
		if (anchor != null ){
			Object bo = getBusinessObjectForPictogramElement(anchor.getParent());
			if (bo instanceof Person) {
				return (Person) bo;
			}
		}
		return null;
	}
	
	private Acquaintance createAcquaintance(Person src, Person trg) {
		Acquaintance acq = SocialnetworkFactory.eINSTANCE.createAcquaintance();
		acq.setSource(src);
		acq.setTarget(trg);
		return acq;
		
	}
}
