package hu.bme.mit.socialnetwork.graphiti.features;

import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.IReason;
import org.eclipse.graphiti.features.context.IUpdateContext;
import org.eclipse.graphiti.features.impl.AbstractUpdateFeature;
import org.eclipse.graphiti.features.impl.Reason;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.pictograms.ContainerShape;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.mm.pictograms.Shape;

import socialnetwork.Person;

public class PersonUpdateFeature extends AbstractUpdateFeature {

	public PersonUpdateFeature(IFeatureProvider fp) {
		super(fp);
	}

	@Override
	public boolean canUpdate(IUpdateContext context) {
		Object bo = getBusinessObjectForPictogramElement(context.getPictogramElement());
		return bo instanceof Person;
	}

	@Override
	public IReason updateNeeded(IUpdateContext context) {
		// Retrieve name from pictogram model
		String shapeText = null;
		PictogramElement pe = context.getPictogramElement();
		if (pe instanceof ContainerShape) {
			ContainerShape cs = (ContainerShape) pe;
			for (Shape s : cs.getChildren()) {
				if (s.getGraphicsAlgorithm() instanceof Text) {
					shapeText = ((Text)s.getGraphicsAlgorithm()).getValue();
				}
			}
		}
		String domainName = getNameFromDomainObject(pe);
		if ((shapeText == null && domainName != null)
				|| (shapeText != null && !shapeText.equals(domainName))) {
			return Reason.createTrueReason("Name has changed");
		}
		return Reason.createFalseReason();
	}

	private String getNameFromDomainObject(PictogramElement pe) {
		// Retrieve name from domain model
		String domainName = null;
		Object bo = getBusinessObjectForPictogramElement(pe);
		if (bo instanceof Person) {
			domainName = ((Person) bo).getName();
		}
		return domainName;
	}

	@Override
	public boolean update(IUpdateContext context) {
		PictogramElement pe = context.getPictogramElement();
		String name = getNameFromDomainObject(pe);
		
		if (pe instanceof ContainerShape) {
			ContainerShape cs = (ContainerShape) pe;
			for (Shape s : cs.getChildren()) {
				if (s.getGraphicsAlgorithm() instanceof Text) {
					((Text)s.getGraphicsAlgorithm()).setValue(name);
				}
			}
		}
		
		return false;
	}

}
